/* 
Object which wraps all view componenets for a particular option. this includes the color, wedge text and input.
*/

var Option = function() {
	var cir = $('#circle');
	this._rgb = [0, 0, 0];
	this.row = $('<div/>')
		.addClass('optionRow');
	this.colorBox = $('<div/>')
		.addClass('optionColorBox')
		.appendTo(this.row);
	this.input = $('<input/>')
		.addClass('optionInput text0')
		.appendTo(this.row);
	this.canvas = $('<canvas/>').css({
		position: 'absolute',
		width: '100%',
		height: '100%',
		pointerEvents: 'none',
	}).appendTo(this.colorBox);
	this.textColor = '';
	this.isTransparent = false;
	this._checkerboardPattern = null;
};

Option.prototype = {

	rgb: function() {
		return this._rgb.slice();
	},

	setRgb: function(rgb) {
		this._rgb = rgb.slice();
		// var style = rgb2Style(rgb); 
		// this.colorBox.css({
		// 	backgroundColor: style,
		// });
	},

	text: function() {
		return this.input.val();
	},

	setText: function(text) {
		this.input.val(text);
	},

	set: function(rgb, text) {
		this.setRgb(rgb);
		this.setText(text);
	},

	caretPos: function() {
		if(this.input.is(':focus')) {
			return this.input[0].selectionStart;
		}
		return -1;
	},

	setCaretPos: function(val) {
		var element = this.input[0];
		if(element.setSelectionRange) {
			element.focus();
			element.setSelectionRange(val, val);
		} else if(element.createTextRange) {
			var range = element.createTextRange();
			range.collapse(true);
			range.moveEnd('character', val);
			range.moveStart('character', val);
			range.select();
		}
	},

	isHalfShown: function() {
		/* True if row is atleast halfway visible in scroll list */
		var list = $('#optionsList');
		var y = $('#optionsListBack').position().top + this.row.position().top + this.row.height() / 2;
		return y > 0 && y < list.height();
	},

	isTransparent: function() {
		return this.isTransparent;
	},

	setIsTransparent: function(isTransparent) {
		this.isTransparent = isTransparent;
	},

	layout: function() {
		var el = this.canvas[0];
		var ctx = el.getContext('2d');
		ctx.beginPath();
		if (this.isTransparent) {
			var pattern = ctx.createPattern(this.getCheckerboardPattern(), "repeat");
			ctx.fillStyle = pattern;
		} else {
			ctx.fillStyle = rgb2Style(this._rgb);
		}
		ctx.fillRect(0, 0, ctx.canvas.width, ctx.canvas.height);
		ctx.fill();
		ctx.stroke();
	},

	getCheckerboardPattern: function() {
		if (this._checkerboardPattern == null) {
			var pattern = document.createElement('canvas');
			pattern.width = 60;
			pattern.height = 60;
			var pctx = pattern.getContext('2d');

			var w=30;
	  		for(i=0; i < 2; i++) {
	    		for(j=0; j < 2; j++) {
	      			y = i * w;
	      			x = j * w;

	        		if(i % 2 != j % 2) {
						pctx.strokeStyle="#C0C0C0";
						pctx.fillStyle = "#C0C0C0";
	      			} else {
						pctx.strokeStyle="#FFFFFF";
						pctx.fillStyle = "#FFFFFF";
	      			}
	      			pctx.fillRect(x,y,w,w);
		        }
		    }

			pctx.fill();

			return pattern;
		} else {
			return this._checkerboardPattern;
		}
	},

};
