var SettingsButton = function(imgSrc) {
	this.jq = $('<div/>')
		.css({
			position: 'absolute',
			overflow: 'hidden',
			borderRadius: 2,
			width: 44,
			height: 20,
		});
	this.back = $('<div/>')	
		.appendTo(this.jq)
		.css({
			position: 'absolute',
			width: '100%',
			height: '100%',
			cursor: 'pointer',
		});
	this.img = $('<img>')
		.appendTo(this.back)
		.attr('src', imgSrc)
		.attr('draggable', false)
		.css({
			position: 'absolute',
			width: 16,
			height: 16,
			top: 2,
			right: 0,
			userSelect: 'none',
		});
	this.animDuration = 300;
	this._finalXPercent = 0;
};

SettingsButton.prototype = {

	layoutBehindCircle: function(circleWidth) {
		//Width / height
		var height = 0.1 * circleWidth;
		if(height > 30) {
			height = 30;
		} else if(height < 16) {
			height = 16;
		}
		var width = 2.7 * height;

		//X, Y
		var radians = -Math.PI / 3;
		var x = circleWidth / 2 + Math.cos(radians) * circleWidth / 2;
		var y = circleWidth / 2 + Math.sin(radians) * circleWidth / 2;
		var right = circleWidth - (x + 0.55 * width);
		var top = y - height * 0.4;
		this.jq.css({
			width: width,
			height: height,
			right: right,
			top: top,
		});

		//Img
		var imgWidth = 0.8 * height;
		this.img.css({
			right: 0.05 * height,
			top: 0.1 * height,
			width: imgWidth,
			height: imgWidth,
		});
		this.updateBorder();
	},

	layoutInCorner: function(width) {
		var height = 0.1 * width;
		if(height > 30) {
			height = 30;
		}
		this.jq.css({
			height: height,
			width: height,
			right: 2,
			top: 2,
		});

		//Img
		var imgWidth = 0.8 * height;
		this.img.css({
			right: 0.1 * height,
			top: 0.1 * height,
			width: imgWidth,
			height: imgWidth,
		});
		this.updateBorder();
	},

	layoutBehindRect: function(rectHeight, top) {
		var height = 0.2 * rectHeight;
		if(height > 30) {
			height = 30;
		} else if(height < 16) {
			height = 16;
		}
		this.jq.css({
			height: height,
			width: height * 1.4,
			right: 0,
			top: top,
		});

		//Img
		var imgWidth = 0.8 * height;
		this.img.css({
			right: 0.1 * height,
			top: 0.1 * height,
			width: imgWidth,
			height: imgWidth,
		});
		this.updateBorder();
	},

	setShown: function(val) {
		this.back.velocity('stop');
		var xPercent = val ? 0 : -100;
		this._finalXPercent = xPercent;
		this.back.css({
			left: xPercent,
		});
	},

	setShownAnim: function(val) {
		// turn off sliding animation if the settings button has a transparent background
		this.animDuration = (this.back.css('background-color') == 'rgba(0, 0, 0, 0)') ? 0 : 300;
		var xPercent = val ? 0 : -100;
		if(this._finalXPercent !== xPercent) {
			this._finalXPercent = xPercent;
			this.back.velocity('stop');
			this.back.velocity({
				left: xPercent + '%',
			}, {
				duration: this.animDuration,
			});
		}
	},

	updateBorder: function() {
		var radius = 0.2 * this.jq.height();
		this.back.css({
			borderRadius: radius,
		});
	},

};
