/* Top level object */

var Spinner = function() {
	this.ready();
};

Spinner.prototype = {

	//Init
	ready: function() {
		this.view = new SpinnerView(this.onOptionInput.bind(this), this.onColorBoxDown.bind(this));
		this.applyStrings();
		this.addListeners();
		this.selectionListener = new SelectionListener(this.onSelectionChange.bind(this));
		this._spinAnim = null;

		//Initialize
		var persist = this.readPersist();
		if(persist) {
			this.view.initialize(persist.settingsShown, 
				persist.options, 
				persist.count, 
				persist.pointerRadians);
		} else {
			var flash = this.readFlash();
			if(flash) {
				this.applyFlash(flash);
			} else {
				this.view.initialize(false, this.defaultOps(), 4, -8.5 * Math.PI / 30);
			}
		}
		this.view.layout();
		this.updateMinSizes();
		this.updatePlusMinusValid();
		$('#spinnerView').css({
			opacity: 1, //Ready to go, so show the view.
		});
	},

	defaultOps: function() {
		return [
			this.defaultOp(255, 148, 57),
			this.defaultOp(34, 194, 220),
			this.defaultOp(219, 143, 219),
			this.defaultOp(26, 162, 212),
			this.defaultOp(72, 209, 103),
			this.defaultOp(251, 117, 199),
			this.defaultOp(130, 137, 188),
			this.defaultOp(62, 185, 157),
			this.defaultOp(225, 68, 91),
			this.defaultOp(0, 221, 221),
			this.defaultOp(23, 126, 151),
			this.defaultOp(44, 207, 51),
		];
	},

	defaultOp: function(r, g, b) {
		return {
			text: '',
			rgb: [r, g, b],
			transparent: false,
		};
	},

	//Persist
	writePersist: function() {
		var data = {
			settingsShown: this.view.settingsShown(),
			options: this.view.opsData(),
			count: this.view.count(),
			pointerRadians: this.view.pointerRadians(),
		};
		NB.persist.save('data', data);
	},

	readPersist: function() {
		var dataString = NB.persist.load('data');
		if(!dataString) {
			return null;
		}
		var json = JSON.parse(dataString);
		var data = {};

		//Ops
		var max = this.maxOps();
		var min = this.minOps();
		data.options = [];
		if(json.options instanceof Array) {
			//Validate length
			if(json.options.length > max) {
				json.options.length = max;
			} else if(json.options.length < min) {
				json.options.length = min;
			}
			//Valildate ops
			for(var i = 0; i < json.options.length; i++) {
				data.options.push(this.validateOption(json.options[i]));
			}
		}

		//Count
		data.count = data.options.length;
		if(typeof json.count === 'number' && json.count % 1 === 0) {
			data.count = json.count;
		}
		if(data.count < min) {
			data.count = min;
		} else if(data.count > data.options.length) {
			data.count = data.options.length
		}

		//Others
		data.settingsShown = Boolean(json.settingsShown);
		var rads = json.pointerRadians;
		if(typeof rads == 'number' && !isNaN(rads) && isFinite(rads)) {
			data.pointerRadians = rads;
		} else {
			data.pointerRadians = 0;
		}
		return data;
	},

	validateOption: function(val) {
		var op = {
			text: '',
			rgb: [0, 0, 0],
			transparent: false,
		}
		if(typeof val === 'object') {
			if(typeof val.text === 'string') {
				op.text = val.text;
			}
			if(typeof val.transparent == 'boolean') {
				op.transparent = val.transparent;
			}
		}
		if(this.isValidRgb(val.rgb)) {
			op.rgb = [val.rgb[0], val.rgb[1], val.rgb[2]];
		}
		return op;
	},

	isValidRgb: function(rgb) {
		if(!rgb) {
			return false;
		}
		for(var i = 0; i < 3; i++) {
			var x = rgb[i];
			if(!(typeof x === 'number' && x >= 0 && x <= 255)) {
				return false;
			}
		}
		return true;
	},

	//Flash
	readFlash: function() {
		var dataString = NB.persist.load('updateData');
		try {
			if(!dataString) {
				return null;
			}
			var perData = new XmlDocument(dataString);
			if(!perData) {
				return null;
			}
			var perProps = this.getPerPropsNode(perData);
			if(!perProps) {
				return null;
			}

			var options = [];
			var count = 2;
			
			//Options
			for(var i = 0; i < 12; i++) {
				options.push(this.getFlashOption(perProps, i));
			}

			//Count
			var segments = perProps.childNamed('segments');
			if(segments) {
				var n = Number(segments.val);
				if(!isNaN(n) && n >= 1 && n <= 12) {
					count = Math.floor(n);
				}
			}
			return {
				options: this.rearrangeFlashOptions(options, count),
				count: count,
			};
		} catch(e) {}
		return null;
	},

	applyFlash: function(flash) {
		this.view.initialize(false, flash.options, flash.count, 0);
		var width = NB.getHostObject().width;
		var min = this.minSizesFor(false).width;
		if(width < min) {
			width = min;
		}
		this.setHostSize(width, width);
	},

	getPerPropsNode: function(persistenceData) {
		var kids = persistenceData.children;
		for(var i = 0; i < kids.length; i++) {
			var k = kids[i];
			if(k.name === 'PersistenceProperties' && k.attr.id === 'spinner') {
				return k;
			}
		}
		return null;
	},

	getFlashOption: function(perProps, index) {
		var text = '';
		var rgb = [120, 120, 120];
		var transparent = false;
		var segText = perProps.childNamed('segText' + index);
		if(segText && segText.val && segText.val !== 'undefined') {
			text = segText.val;
		}
		var segColor = perProps.childNamed('segColor' + index);
		if(segColor) {
			if (segColor.val == '10066329') {
				rgb = [255, 255, 255];
				transparent = true;
			} else {
				rgb = this.parseFlashColor(segColor.val);
			}
		}
		return {
			text: text,
			rgb: rgb,
			transparent: transparent,
		};
	},

	parseFlashColor: function(text) {
		var hasX = text.indexOf('x') !== -1;
		var r = 0;
		var g = 0;
		var b = 0;
		var n = Number(text)
		if(isNaN(n)) {
			return [0, 0, 0];
		} else {
			var r = Math.floor(n / 0x10000);
			if(r > 255 || r < 0) {
				r = 0;
			}
			var g =  Math.floor((n % 0x10000) / 0x100);
			if(g > 255 || g < 0) {
				g = 0;
			}
			var b = Math.floor(n % 0x100);
			if(b > 255 || b < 0) {
				b = 0;
			}
			return [r, g, b];
		}
	},

	rearrangeFlashOptions: function(ops, count) {
		/* Flash Options are stored in a different order than our options. 
			 - The first option for the flash widget is the top right segment and then the
			index increases going counter clockwise. 
			- The first option for the html widget is the bottom left and then the index 
			increases going clockwise. */
		var newOps = ops.slice();
		if(count === 2 || count === 3) {
			newOps[0] = ops[1];
			newOps[1] = ops[0];
		} else {
			//Walk CCW from top right
			var newIndex = Math.floor(count / 2);
			for(var i = 0; i < count; i++) {
				newOps[newIndex] = ops[i];
				newIndex--;
				if(newIndex < 0) {
					newIndex+= count;
				}
			}

		}
		return newOps;
	},

	//Vars
	maxChars: function() {
		return 15;
	},

	maxOps: function() {
		return 12;
	},

	minOps: function() {
		return 1;
	},

	//Toggle Settings
	toggleSettings: function() {
		new ToggleSettings(this, !this.view.settingsShown());
	},

	setSettingsShown: function(val) {
		if(val === this.view.settingsShown()) {
			return;
		}
		var x = new FadedAction({
			view: $('#spinnerView'),
			onMid: this._setSettingsShownFaded.bind(this),
			onEnd: this.updatePreview.bind(this),
			props: {
				show: val,
			},
		});
		x.start();
	},

	_setSettingsShownFaded: function(action) {
		this._setSettingsShown(action.props.show);
	},

	_setSettingsShown: function(val) {
		var circle = $('#circle');
		var percentMarg = this.view.circleMarg();
		var marg = circle.width() * percentMarg / (1 - 2 * percentMarg);
		var oldWidth = $('#spinnerView').width();
		var oldHeight = $('#spinnerView').height();
		var newWidth = oldWidth;
		var newHeight = oldHeight;
		var dx = 0;
		if(val) {
			dx = -marg;
			newWidth += 2 * marg;
			newHeight = 1.65 * newWidth;
		} else {
			dx = marg;
			newWidth -= 2 * marg;
			newHeight = newWidth;
		}
		if(NB.getHostObject().rotation !== 0) {
			dx = 0;
		}
		this.offsetHostRect(dx, dx, newWidth - oldWidth, newHeight - oldHeight);
		this.view.setSettingsShown(val);
		this.updateMinSizes();
		this.writePersist();
		this.updatePreview();
	},

	setHostSize: function(w, h) {
		NB.undomanager.setAutoRecord(false);
		var host = NB.getHostObject();
		NB.batch(function() {
			host.width = w
			host.height = h;
		});
		NB.undomanager.setAutoRecord(true);
	},

	offsetHostRect: function(dx, dy, dw, dh) {
		NB.undomanager.setAutoRecord(false);
		var host = NB.getHostObject();
		NB.batch(function() {
			host.x = host.x + dx;
			host.y = host.y + dy;
			host.width = host.width + dw;
			host.height = host.height + dh;
		});
		NB.undomanager.setAutoRecord(true);
	},

	//Spin
	spin: function() {
		if(this._spinAnim) {
			this._spinAnim.stop();
		}
		this._spinAnim = new SpinAnim(this.view, this._onSpinEnd.bind(this));
	},

	isSpinning: function() {
		return this._spinAnim != null;
	},

	_onSpinEnd: function() {
		this._spinAnim = null;
		this.writePersist();
		this.updatePreview();
	},

	//Events
	addListeners: function() {
		$('#optionsList').scroll(this.onListScroll.bind(this));
		$('#plus').click(this.onPlus.bind(this));
		$('#minus').click(this.onMinus.bind(this));
		$('#spinnerView').mousedown(this.onMainDown.bind(this));
		$('#circleInner, #pointerCircle, #pointerLine').click(this.onWheelClick.bind(this));
		$(window).resize(this.onResize.bind(this))
			.focusout(this.onFocusout.bind(this))
			.focusin(this.onFocusin.bind(this));
		this.view.settingsBtn.back.click(this.onSettingsButton.bind(this)); //Attach to back for larger touch target
		var cp = this.view.picker;
		cp.onHexClick = this.onPickerHexClick.bind(this);
		cp.onWheelUp = this.onPickerWheelUp.bind(this);
		cp.onDrag = this.onPickerDrag.bind(this);
		cp.onXClick = this.onPickerXClick.bind(this);
		$('#spinnerView').on('mousewheel', this.onWheel.bind(this));
		$('#optionsList').on('mousewheel', this.onListWheel.bind(this));
	},

	onWheel: function(event) {
		var dy = -event.originalEvent.wheelDelta;
		var list = $('#optionsList');
		list.scrollTop(list.scrollTop() + dy);
		
	},

	onListWheel: function(event) {
		event.stopPropagation();
	},

	onOptionInput: function(event, option) {
		var newVal = option.input.val();
		var max = this.maxChars();
		if(newVal.length > max) {
			newVal = newVal.substring(0, max);
			var caret = option.caretPos();
			if(caret > max) {
				caret = max;
			}
			option.input.val(newVal);
			option.setCaretPos(caret);
		}
		this.view.drawWedges();
		this.writePersist();
		this.updatePreview();
	},

	onColorBoxDown: function(event, option) {
		event.stopPropagation();
		this.view.togglePicker(option);
		this.updatePreview();
	},

	onListScroll: function() {
		this.view.updatePickerPos();
	},

	onPlus: function() {
		var n = this.view.count();
		if(n < this.maxOps()) {
			this.view.addOption();
			this.updatePlusMinusValid();
			this.writePersist();
			this.updatePreview();
		}
	},

	onMinus: function() {
		var n = this.view.count();
		if(n > this.minOps()) {
			this.view.removeOption();
			this.updatePlusMinusValid();
			this.writePersist();
			this.updatePreview();
		}
	},

	onMainDown: function() {
		this.view.setPickerIndex(-1);
		this.updatePreview();
	},

	onSettingsButton: function() {
		this.toggleSettings();
	},

	onResize: function() {
		this.view.layout();
		setTimeout(this._onResize.bind(this), 10); //In the case where improper window coords are sent to Widget
	},

	_onResize: function() {
		this.view.layout();
	},

	onFocusout: function() {
		if(!this.view.settingsShown() && !this.selectionListener.isSelected()) {
			this.view.settingsBtn.setShownAnim(false);
		}
	},

	onFocusin: function() {
		this.view.settingsBtn.setShownAnim(true);
	},

	onWheelClick: function(event) {
		if(!this.isSpinning()) {
			var cir = $('#circle');
			var radius = cir.width() / 2
			var off = cir.offset();
			var x = event.pageX - off.left - radius;
			var y = event.pageY - off.top - radius;
			var isInBounds = x * x + y * y < radius * radius;
			if(isInBounds) {
				event.stopPropagation();
				this.spin();
			}
		}	
	},

	//Color Picker Events
	onPickerHexClick: function(picker, newRgb, _isTransparent) {
		this.updateTransparency(_isTransparent);
		this.updatePointerColor();
		this.updateRgb(newRgb);
		this.layout();
		this.writePersist();
		this.updatePreview();
	},

	onPickerWheelUp: function(picker, newRgb) {
		this.writePersist();
		this.updatePreview();
	},

	onPickerDrag: function(picker, newRgb) {
		this.updateRgb(newRgb);
	},

	onPickerXClick: function(picker) {
		this.view.setPickerIndex(-1);
		this.updatePreview();
	},

	updateRgb: function(rgb) {
		var op = this.view.options[this.view.pickerIndex()];
		this.view.setOptionRgb(op, rgb);
	},

	updateTransparency: function(isTransparent) {
		var op = this.view.options[this.view.pickerIndex()];
		this.view.setOptionTransparency(op, isTransparent);
	},

	layout: function() {
		var op = this.view.options[this.view.pickerIndex()];
		this.view.layout();
	},

	updatePointerColor: function() {
		this.view.updatePointerColor();
	},

	//NB
	updatePreview: function() {
		// add delay to give nb some time to update its internal bitmaps
		setTimeout(function() {
			NB.page.updateThumbnail();
		}, 100);
	},

	updateMinSizes: function() {
		var mins = this.minSizesFor(this.view.settingsShown());
		NB.getHostObject().setWidgetMinSize(mins.width, mins.height);
	},

	minSizesFor: function(settingsShown) {
		if(settingsShown) {
			return {
				width: 184,
				height: 368,
			};
		} else {
			return {
				width: 150,
				height: 150,
			};
		}
	},

	//Other
	onSelectionChange: function(listener, isSelected) {
		if(!this.view.settingsShown()) {
			this.view.settingsBtn.setShownAnim(isSelected);
		}
	},

	updatePlusMinusValid: function() {
		var n = this.view.count();
		this.view.setPlusMinusValid(n < this.maxOps(), n > this.minOps());
	},

	applyStrings: function() {
		var segments = NB.localize('Segments');
		var option = NB.localize('Add text');
		$('#segments').text(segments);
		this.view.setPlaceholder(option);
	},

};

NB.ready(function() {
	if (window.requiresNewerNBSDKVersionError) {
        var container = window.document.createElement("div");
        container.setAttribute("id", "requiresNewerNBSDKVersionErrorContainer");

        var p1 = window.document.createElement("p");
        p1.textContent = NB.localize("This activity requires a later version of SMART Notebook software.");
        container.appendChild(p1);

        var p2 = window.document.createElement("p");
        p2.textContent = NB.localize("Make sure you're using Notebook software version %d or later.").replace("%d", "16");
        container.appendChild(p2);

        document.body.appendChild(container);
        document.body.style.backgroundColor = "lightgray";

        function adjustHeights(id) {
            var elem = document.getElementById(id);
            var fontstep = 2;
            if (elem.offsetHeight > document.documentElement.clientHeight) {
                elem.style.fontSize = ((window.getComputedStyle(elem).fontSize.substr(0,2)-fontstep).toString() + 'px');
                elem.style.lineHeight = (window.getComputedStyle(elem).fontSize.substr(0,2).toString() + 'px');
                adjustHeights(id);
            }
        }

        adjustHeights("requiresNewerNBSDKVersionErrorContainer");

        return;
    }
	x = new Spinner();
});

